@@ -8,27 +8,30 @@ Title: tn93.py
 Description: Implementation of Tamura-Nei distance calculation for pair of HIV sequences
 Usage: Used by other software
 Date Created: 2022-08-09 18:11
-Last Modified: Thu 15 Sep 2022 10:13:08 PM EDT
+Last Modified: Thu 29 Sep 2022 10:54:49 AM EDT
 Author: Reagan Kelly (ylb9@cdc.gov)
 """
 
 import copy
+import string
+import random
 import csv
-import re
-import argparse
 import json
-import sys
 import logging
 import math
+import re
+import sys
+from itertools import chain
 from Bio import SeqIO
 import numpy as np
+import argparser
 
 
 def main(args):
-    parser = setup_parser()
+    parser = argparser.setup_parser()
     args = parser.parse_args(args)
     tn93 = TN93(
-        show_counts=args.show_counts,
+        verbose=args.verbose,
         ignore_gaps=args.ignore_terminal_gaps,
         json_output=args.json_output,
         max_ambig_fraction=args.max_ambig_fraction,
@@ -68,64 +71,10 @@ def main(args):
             writer.writerows(final_distance)
 
 
-def setup_parser():
-    parser = argparse.ArgumentParser()
-    parser.add_argument(
-        "-i",
-        "--input_file",
-        action="store",
-        required=True,
-        help="Path to the input fasta file",
-    )
-    parser.add_argument(
-        "-m",
-        "--match_mode",
-        action="store",
-        required=True,
-        help="""
-        How to handle ambiguities. This can be one of four options:
-        average - Averages the possible nucleotide values for each ambiguity in a sequence;
-        resolve - Tries to resolve ambiguities;
-        skip - Ignores gaps and ambiguities;
-        gapmm - Treats gaps in only one sequence as 'N's;
-        """,
-    )
-    parser.add_argument(
-        "-o",
-        "--output",
-        action="store",
-        required=True,
-        help="The name of the output file to create",
-    )
-    # parser.add_argument("-g", "--max_ambig_fraction", action="store", required=False, default=1.0, help="Sequences that have proportions of ambiguities lower than this value will be resolved, otherwise they will be averaged (Default: 1.0)")
-    parser.add_argument(
-        "-c",
-        "--show_counts",
-        action="store_true",
-        default=False,
-        help="Show counts and other debugging information (Default: False)",
-    )
-    parser.add_argument(
-        "-n",
-        "--ignore_terminal_gaps",
-        action="store_true",
-        default=False,
-        help="Should gaps at the beginning and end of a sequence be ignored (GAPMM only)? (Default: False)",
-    )
-    parser.add_argument(
-        "-j",
-        "--json_output",
-        action="store_true",
-        default=False,
-        help="Should the output be in JSON format? (Default: False)",
-    )
-    return parser
-
-
 class TN93(object):
     def __init__(
         self,
-        show_counts=False,
+        verbose=False,
         ignore_gaps=False,
         json_output=False,
         max_ambig_fraction=1.0,
@@ -135,7 +84,7 @@ class TN93(object):
             self.resolutions,
             self.resolutionsCount,
         ) = self.get_constants()
-        self.show_counts = show_counts
+        self.verbose = verbose
         self.ignore_gaps = ignore_gaps
         self.json_output = json_output
         self.max_ambig_fraction = max_ambig_fraction
@@ -151,7 +100,7 @@ class TN93(object):
             seq_ids = [1, 2]
         nucleotide_frequency = self.get_nucleotide_frequency(pairwise_counts)
         dist = self.get_distance(pairwise_counts, nucleotide_frequency)
-        if self.show_counts:
+        if self.verbose > 0:
             logging.error(dist)
         if self.json_output:
             output = {"ID1": seq_ids[0], "ID2": seq_ids[1], "Distance": dist}
@@ -160,13 +109,13 @@ class TN93(object):
         return output
 
     def get_distance(self, pairwise_counts, nucleotide_frequency):
-        if self.show_counts:
+        if self.verbose > 0:
             logging.error(pairwise_counts[0])
             logging.error(pairwise_counts[1])
             logging.error(pairwise_counts[2])
             logging.error(pairwise_counts[3])
             logging.error(nucleotide_frequency)
-            logging.error(sum([sum(x) for x in pairwise_counts]))
+            logging.error(sum(nucleotide_frequency) / 2)
         dist = 0
         total_non_gap = 2 / sum(nucleotide_frequency)
         AG_counts = float(pairwise_counts[0][2] + pairwise_counts[2][0])
@@ -180,7 +129,7 @@ class TN93(object):
             + pairwise_counts[3][3]
         ) * total_non_gap
         tv = 1 - (AG + CT + matching)
-        if self.show_counts:
+        if self.verbose > 0:
             logging.error(f"Initially: AG={AG} CT={CT} tv={tv}")
         if 0 in nucleotide_frequency:
             AG = 1 - 2 * (AG + CT) - tv
@@ -208,7 +157,7 @@ class TN93(object):
                 -K1 * math.log(AG) - K2 * math.log(CT) - K3 * math.log(tv)
             )
 
-        if self.show_counts:
+        if self.verbose > 0:
             logging.error(
                 f"Finally: auxd={auxd} fR={fR} fY={fY} K1={K1} K2={K2} K3={K3} AG={AG} CT={CT} tv={tv}"
             )
@@ -231,15 +180,24 @@ class TN93(object):
         e2 = re.search(r"-*$", seq2).start()
         self.first_nongap = max(s1, s2)
         self.last_nongap = min(e1, e2) - 1
-        if self.show_counts:
+        if self.verbose > 0:
             logging.error(
                 f"{self.first_nongap}-{seq1[self.first_nongap]},{seq2[self.first_nongap]} {self.last_nongap}-{seq1[self.last_nongap]},{seq2[self.last_nongap]}"
             )
 
+    def ambig_fraction_too_high(self, seq):
+        ambig_count = len([x for x in seq if x not in ["A", "C", "G", "T", "U", "-"]])
+        if (ambig_count / len(seq)) > self.max_ambig_fraction:
+            return True
+        return False
+
     def get_counts(self, seq1, seq2, match_mode):
         self.find_terminal_gaps(seq1, seq2)
         if match_mode.upper() == "RESOLVE":
-            pairwise_counts = self.get_counts_resolve(seq1, seq2)
+            if self.ambig_fraction_too_high(seq1) or self.ambig_fraction_too_high(seq2):
+                pairwise_counts = self.get_counts_average(seq1, seq2)
+            else:
+                pairwise_counts = self.get_counts_resolve(seq1, seq2)
         elif match_mode.upper() == "AVERAGE":
             pairwise_counts = self.get_counts_average(seq1, seq2)
         elif match_mode.upper() == "GAPMM":
@@ -253,11 +211,12 @@ class TN93(object):
 
     def round(self, number):
         val = np.format_float_positional(
-            number, precision=4, unique=False, fractional=False, trim="k"
+            number, precision=6, unique=False, fractional=False, trim="k"
         )
         return float(val)
 
     def get_counts_resolve(self, seq1, seq2):
+        all_pairwise_count_arrays = {}
         length = min(len(seq1), len(seq2))
         pairwise_counts = [
             # A, C, G, T
@@ -270,7 +229,7 @@ class TN93(object):
             nuc1 = self.map_character[ord(seq1[p])]
             nuc2 = self.map_character[ord(seq2[p])]
             if nuc1 < 4 and nuc2 < 4:  # If neither nucleotide is ambiguous
-                if self.show_counts and nuc1 == 3 and nuc2 == 1:
+                if self.verbose and nuc1 == 3 and nuc2 == 1:
                     logging.error(f"{p} is TC")
                 pairwise_counts[nuc1][nuc2] += 1
             else:
@@ -316,9 +275,21 @@ class TN93(object):
                                 for k in range(4):
                                     if self.resolutions[nuc2][k]:
                                         pairwise_counts[j][k] += norm
+            all_pairwise_count_arrays[str(p)] = copy.deepcopy(pairwise_counts)
+        if self.verbose > 1:
+            count_holder = []
+            for k in all_pairwise_count_arrays.keys():
+                count_line = list(chain.from_iterable(all_pairwise_count_arrays[k]))
+                final_line = [int(k)] + count_line
+                count_holder += [final_line]
+            filename_slug = self.make_filename()
+            with open(f"{filename_slug}_python_resolve_counts.csv", "w") as cf:
+                writer = csv.writer(cf)
+                writer.writerows(count_holder)
         return pairwise_counts
 
     def get_counts_average(self, seq1, seq2):
+        all_pairwise_count_arrays = {}
         length = min(len(seq1), len(seq2))
         pairwise_counts = [
             # A, C, G, T
@@ -353,9 +324,21 @@ class TN93(object):
                                 for k in range(4):
                                     if self.resolutions[nuc2][k]:
                                         pairwise_counts[j][k] += norm
+            all_pairwise_count_arrays[str(p)] = copy.deepcopy(pairwise_counts)
+        if self.verbose > 1:
+            count_holder = []
+            for k in all_pairwise_count_arrays.keys():
+                count_line = list(chain.from_iterable(all_pairwise_count_arrays[k]))
+                final_line = [int(k)] + count_line
+                count_holder += [final_line]
+            filename_slug = self.make_filename()
+            with open(f"{filename_slug}_python_avg_counts.csv", "w") as cf:
+                writer = csv.writer(cf)
+                writer.writerows(count_holder)
         return pairwise_counts
 
     def get_counts_gapmm(self, seq1, seq2):
+        all_pairwise_count_arrays = {}
         length = min(len(seq1), len(seq2))
         pairwise_counts = [
             # A, C, G, T
@@ -401,6 +384,17 @@ class TN93(object):
                                     for k in range(4):
                                         if self.resolutions[nuc2][k]:
                                             pairwise_counts[j][k] += norm
+            all_pairwise_count_arrays[str(p)] = copy.deepcopy(pairwise_counts)
+        if self.verbose > 1:
+            count_holder = []
+            for k in all_pairwise_count_arrays.keys():
+                count_line = list(chain.from_iterable(all_pairwise_count_arrays[k]))
+                final_line = [int(k)] + count_line
+                count_holder += [final_line]
+            filename_slug = self.make_filename()
+            with open(f"{filename_slug}_python_gapmm_counts.csv", "w") as cf:
+                writer = csv.writer(cf)
+                writer.writerows(count_holder)
         return pairwise_counts
 
     def get_counts_skip(self, seq1, seq2):
@@ -420,9 +414,23 @@ class TN93(object):
             nuc2 = self.map_character[ord(seq2[p])]
             if nuc1 < 4 and nuc2 < 4:  # If neither nucleotide is ambiguous
                 pairwise_counts[nuc1][nuc2] += 1
-            all_pairwise_count_arrays[p] = copy.deepcopy(pairwise_counts)
+            all_pairwise_count_arrays[str(p)] = copy.deepcopy(pairwise_counts)
+        if self.verbose > 1:
+            count_holder = []
+            for k in all_pairwise_count_arrays.keys():
+                count_holder += [p] + [
+                    list(chain.from_iterable(all_pairwise_count_arrays[k]))
+                ]
+            filename_slug = self.make_filename()
+            with open(f"{filename_slug}_python_skip_counts.csv", "w") as cf:
+                writer = csv.writer(cf)
+                writer.writerows(count_holder)
+
         return pairwise_counts
 
+    def make_filename(self, N=8):
+        return "".join(random.choices(string.ascii_uppercase + string.digits, k=N))
+
     def get_constants(self):
         map_character = [16 for x in range(256)]
         map_character[45] = 17  # GAP
